
struct NullParameter;
namespace NSVarTypeDict
{
template <size_t N, template<typename...> class TCont, typename...T>
struct _Create
{
	using type = typename _Create<N-1,TCont, NullParameter,T...>::type;
};
template<template<typename...> class TCont, typename...T>
struct _Create<0, TCont, T...>
{
	using type = TCont<T...>;
};

template <size_t Counter, typename TTag, typename THead, typename...TTails>
constexpr size_t Tag2IdImpl()
{
	if (std::is_same<TTag, THead>::value)
	{
		return Counter;
	}
	else
	{
		if constexpr(sizeof...(TTails) > 0)
		{
			return Tag2IdImpl<Counter+1, TTag, TTails...>();
		}
		else
		{
			//deliberately return an invalid index so that returning result can be verified by parameter size
			return Counter+1;
		}
	}
}

template<typename TTag, typename...TParameter>
constexpr size_t Tag2ID()
{
	return Tag2IdImpl<0, TTag, TParameter...>();
}

template<size_t N, typename TCur, typename...TRemains>
struct Pos2TypeImpl
{
	using type=typename Pos2TypeImpl<N-1, TRemains...>::type;
};
template<typename TCur, typename...TRemains>
struct Pos2TypeImpl<0, TCur, TRemains...>
{
	using type=TCur;
};
template<size_t N, typename...TTypes>
struct Pos2Type
{
	using type=typename Pos2TypeImpl<N, TTypes...>::type;
};

template<typename TVal, size_t N, size_t M, typename TProcessedTypes, typename...TRemainTypes>
struct NewTupleType_;

template<typename TVal, size_t N, size_t M, template<typename...> class TCont,
		typename...TProcessedTypes, typename TCurType, typename...TRemainTypes>
struct NewTupleType_<TVal, N, M, TCont<TProcessedTypes...>,TCurType, TRemainTypes... >
{
	using type = typename NewTupleType_<TVal, N, M+1, TCont<TProcessedTypes..., TCurType>, TRemainTypes...>::type;
};

template<typename TVal, size_t N, template<typename...> class TCont,
		typename...TProcessedTypes, typename TCurType, typename...TRemainTypes>
struct NewTupleType_<TVal, N, N, TCont<TProcessedTypes...>,TCurType, TRemainTypes... >
{
	using type = TCont<TProcessedTypes..., TVal, TRemainTypes...>;
};

template<typename TVal, size_t TagPos, typename TCont, typename...TRemainTypes>
using NewTupleType = typename NewTupleType_<TVal, TagPos, 0, TCont, TRemainTypes...>::type;
}
template<typename...TParamters>
struct VarTypeDict
{
public:
	template<typename...TTypes>
	struct Values
	{
		Values() = default;
		Values(std::shared_ptr<void>(&& input)[sizeof...(TTypes)])
		{
			for (size_t i = 0; i < sizeof...(TTypes); ++ i)
			{
				m_tuple[i] = std::move(input[i]);
			}
		}
		template<typename TTag, typename TVal>
		auto Set(TVal&& val) &&
		{
			using namespace NSVarTypeDict;
			constexpr static size_t tagPos = Tag2ID<TTag, TParamters...>();
			using RawType = typename std::decay<TVal>::type;
			RawType* tmp = new RawType(std::forward<TVal>(val));
			m_tuple[tagPos] = std::shared_ptr<void>(tmp, [](void*ptr)
					{
						RawType* nPtr= static_cast<RawType*>(ptr);
						delete nPtr;
					});
			using new_type=NewTupleType<RawType, tagPos, Values<>, TTypes...>;
			return new_type(std::move(m_tuple));
		}
		template<typename TTag>
		const auto& Get() const
		{
			using namespace NSVarTypeDict;
			constexpr static size_t tagPos = Tag2ID<TTag, TParamters...>();
			using AimType=typename Pos2Type<tagPos, TTypes...>::type;
			void* ptr = m_tuple[tagPos].get();
			AimType* ref = static_cast<AimType*>(ptr);
			return *ref;
		}
	private:
		std::shared_ptr<void> m_tuple[sizeof...(TTypes)];
	};
	static auto Create()
	{
		using namespace NSVarTypeDict;
		using type = typename _Create<sizeof...(TParamters), Values>::type;
		return type{};
	}
private:
};
